@@ -74,25 +74,24 @@ to a title or subtitle in the document.
 '''
 
 import re
-import string
 import itertools
 import logging
-
-import types
 import collections
 
-from zim.newfs import LocalFile
-from zim.parsing import link_type, is_url_re, is_www_link_re, \
-	url_encode, url_decode, URL_ENCODE_READABLE, URL_ENCODE_DATA
-from zim.parser import Builder
-from zim.config import data_file, ConfigDict
+from functools import reduce
+
+logger = logging.getLogger('zim.formats')
+
+
+from zim.parse.encode import url_decode, url_encode, URL_ENCODE_READABLE, URL_ENCODE_DATA
+from zim.parse.links import link_type, is_url_re, is_www_link_re
+from zim.parse.tokenlist import TokenParser, topLevelLists, collect_until_end_token
+from zim.parse.builder import Builder
+
+from zim.config import ConfigDict
 from zim.plugins import PluginManager
 
 import zim.plugins
-from functools import reduce
-
-
-logger = logging.getLogger('zim.formats')
 
 # Needed to determine RTL, but may not be available
 # if gtk bindings are not installed
@@ -100,7 +99,7 @@ try:
 	from gi.repository import Pango
 except:
 	Pango = None
-	logger.warn('Could not load pango - RTL scripts may look bad')
+	logger.warning('Could not load pango - RTL scripts may look bad')
 
 import xml.etree.ElementTree # needed to compile with cElementTree
 try:
@@ -300,19 +299,14 @@ TokenListElement = collections.namedtuple('TokenListElement', ('tag', 'attrib',
 class ParseTree(object):
 	'''Wrapper for zim parse trees.'''
 
-	# No longer derives from ElementTree, internals are now private
-
-	# TODO, also remove etree args from init
-	# TODO, rename to FormattedText
-
 	def __init__(self, *arg, **kwarg):
 		self._etree = ElementTreeModule.ElementTree(*arg, **kwarg)
 		self._object_cache = {}
-		self.meta = DefinitionOrderedDict()
+		self.meta = LastDefinedOrderedDict()
+		assert not (self._etree.getroot() and self._etree.getroot().attrib.get('raw', False)), 'Deprecated "raw" attribute'
 
 	@classmethod
 	def new_from_tokens(klass, tokens):
-		from zim.tokenparser import TokenParser
 		tokens = list(tokens) # TODO: allow efficient use of generator here ?
 		assert tokens
 		if tokens[0][0] != FORMATTEDTEXT:
@@ -332,13 +326,6 @@ class ParseTree(object):
 			bool(list(root)) or (root.text and not root.text.isspace())
 		)
 
-	@property
-	def israw(self):
-		'''Returns True when this is a raw tree (which is representation
-		of TextBuffer, but not really valid).
-		'''
-		return self._etree.getroot().attrib.get('raw', False)
-
 	def _set_root_attrib(self, key, value):
 		self._etree.getroot().attrib[key] = value
 
@@ -397,8 +384,6 @@ class ParseTree(object):
 		return ParseTree().fromstring(self.tostring())
 
 	def iter_tokens(self):
-		from zim.tokenparser import topLevelLists
-
 		return iter(topLevelLists(self._get_tokens(self._etree.getroot())))
 
 	def _get_tokens(self, node):
@@ -537,7 +522,7 @@ class ParseTree(object):
 
 	def encode_urls(self, mode=URL_ENCODE_READABLE):
 		'''Calls encode_url() on all links that contain urls.
-		See zim.parsing for details. Modifies the parse tree.
+		See zim.parse.links for details. Modifies the parse tree.
 		'''
 		for link in self._etree.iter('link'):
 			href = link.attrib['href']
@@ -548,7 +533,7 @@ class ParseTree(object):
 
 	def decode_urls(self, mode=URL_ENCODE_READABLE):
 		'''Calls decode_url() on all links that contain urls.
-		See zim.parsing for details. Modifies the parse tree.
+		See zim.parse.links for details. Modifies the parse tree.
 		'''
 		for link in self._etree.iter('link'):
 			href = link.attrib['href']
@@ -610,8 +595,6 @@ class ParseTree(object):
 
 	def iter_elements(self, tag):
 		'''Helper function to find all occurences of C{tag}, yields L{TokenListElement}s'''
-		from zim.tokenparser import collect_until_end_token
-
 		token_iter = self.iter_tokens()
 		for t in token_iter:
 			if t[0] == tag:
@@ -628,8 +611,6 @@ class ParseTree(object):
 		C{tags}. The return value of C{func} can be C{None} to remove the element,
 		the same or a modified L{TokenListElement} or a list of tokens.
 		'''
-		from zim.tokenparser import collect_until_end_token
-
 		tokens = []
 		token_iter = self.iter_tokens()
 		for t in token_iter:
@@ -653,11 +634,9 @@ class ParseTree(object):
 def split_heading_from_parsetree(parsetree, keep_head_token=True):
 	'''Helper function to split the header from a L{ParseTree}
 	Looks for a header at the start of a page and strips empty lines after it.
-	Returns two L{ParseTree} objects: one for the header (can be C{None}) and
-	one for the main body of the content.
+	Returns two L{ParseTree} objects: one for the header and one for the main
+	body of the content - both can be C{None} if they are empty.
 	'''
-	from zim.tokenparser import collect_until_end_token
-
 	token_iter = parsetree.iter_tokens()
 	heading = []
 	body = []
@@ -687,17 +666,17 @@ def split_heading_from_parsetree(parsetree, keep_head_token=True):
 	if body[-1] == (END, FORMATTEDTEXT):
 		body.pop()
 
-	if heading:
-		if not keep_head_token:
-			heading = heading[1:-1]
-			if heading[-1][0] == TEXT:
-				if heading[-1][1] == '\n':
-					heading.pop()
-				elif heading[-1][1].endswith('\n'):
-					heading[-1] = (TEXT, heading[-1][1][:-1])
-		return (ParseTree.new_from_tokens(heading), ParseTree.new_from_tokens(body))
-	else:
-		return (None, ParseTree.new_from_tokens(body))
+	if heading and not keep_head_token:
+		heading = heading[1:-1]
+		if heading[-1][0] == TEXT:
+			if heading[-1][1] == '\n':
+				heading.pop()
+			elif heading[-1][1].endswith('\n'):
+				heading[-1] = (TEXT, heading[-1][1][:-1])
+
+	heading_tree = ParseTree.new_from_tokens(heading) if heading else None
+	body_tree = ParseTree.new_from_tokens(body) if body else None
+	return heading_tree, body_tree
 
 
 class ParseTreeBuilder(Builder):
@@ -722,7 +701,7 @@ class ParseTreeBuilder(Builder):
 		self.stack.append(tag)
 		if tag in BLOCK_LEVEL:
 			if self._last_char and self._last_char != '\n':
-				logger.warn('Missing "\\n" before new block (%s)' % tag)
+				logger.warning('Missing "\\n" before new block (%s)' % tag)
 			self._last_char = None
 
 	def text(self, text):
@@ -910,7 +889,7 @@ class ParserClass(object):
 			attrib = {'src': url[:i]}
 			for option in url[i + 1:].split('&'):
 				if option.find('=') == -1:
-					logger.warn('Mal-formed options in "%s"', url)
+					logger.warning('Mal-formed options in "%s"', url)
 					break
 
 				k, v = option.split('=', 1)
@@ -919,7 +898,7 @@ class ParserClass(object):
 						value = url_decode(v, mode=URL_ENCODE_DATA)
 						attrib[str(k)] = value # str to avoid unicode key
 				else:
-					logger.warn('Unknown attribute "%s" in "%s"', k, url)
+					logger.warning('Unknown attribute "%s" in "%s"', k, url)
 			return attrib
 		else:
 			return {'src': url}
@@ -1245,7 +1224,7 @@ class TableParser():
 		Each cell in a list of rows is split by "\n" and a 3-dimensional list is returned,
 		whereas each tuple represents a line and multiple lines represents a row and multiple rows represents the table
 		c11a = Cell in Row 1 in Column 1 in first = a line
-		:param strings: format like (('c11a \n c11b', 'c12a \n c12b'), ('c21', 'c22a \n 22b'))
+		:param rows: format like (('c11a \n c11b', 'c12a \n c12b'), ('c21', 'c22a \n 22b'))
 		:return: format like (((c11a, c12a), (c11b, c12b)), ((c21, c22a), ('', c22b)))
 		'''
 		multi_rows = [[cell.split("\n") for cell in row] for row in rows]
@@ -1310,6 +1289,7 @@ class TableParser():
 		:param row: tuple of cells
 		:param maxwidths: list of column length
 		:param aligns:  list of alignments
+		:param wraps: 0 for not wrapped, 1 for auto-wrapped line display
 		:param x:  point-separator
 		:param y: space-separator
 		:return: a textline
@@ -1362,10 +1342,10 @@ class TableParser():
 		return cells
 
 
-from zim.config.dicts import DefinitionOrderedDict
+from zim.base import LastDefinedOrderedDict
 
-_is_header_re = re.compile('^([\w\-]+):\s+(.*?)\n', re.M)
-_is_continue_re = re.compile('^([^\S\n]+)(.+?)\n', re.M)
+_is_header_re = re.compile(r'^([\w\-]+):\s+(.*?)\n', re.M)
+_is_continue_re = re.compile(r'^([^\S\n]+)(.+?)\n', re.M)
 
 def parse_header_lines(text):
 	'''Read header lines in the rfc822 format.
@@ -1378,7 +1358,7 @@ def parse_header_lines(text):
 	@returns: the text minus the headers and a dict with the headers
 	'''
 	assert isinstance(text, str)
-	meta = DefinitionOrderedDict()
+	meta = LastDefinedOrderedDict()
 	match = _is_header_re.match(text)
 	pos = 0
 	while match:
