@@ -18,13 +18,13 @@ import os
 import re
 import sys
 import shutil
-import tempfile
 import errno
 import logging
 
 
+
 from zim.errors import Error
-from zim.parsing import url_encode, url_decode, URL_ENCODE_READABLE
+from zim.parse.encode import url_decode, url_encode
 from zim.signals import SignalEmitter, SIGNAL_AFTER
 
 logger = logging.getLogger('zim.fs')
@@ -165,7 +165,7 @@ def normalize_file_uris(path):
 
 
 def normalize_win32_share(path):
-	'''Translates paths for windows shares in the platform specific
+	r'''Translates paths for windows shares in the platform specific
 	form. So on windows it translates C{smb://} URLs to C{\\host\share}
 	form, and vice versa on all other platforms.
 	Just returns the original path if it was already in the right form,
@@ -1238,7 +1238,7 @@ class File(FilePath):
 					raise
 
 			if not self._mtime == mtime:
-				logger.warn('mtime check failed for %s, trying md5', self.path)
+				logger.warning('mtime check failed for %s, trying md5', self.path)
 				if self._md5 != _md5(self._read()):
 					raise FileWriteError(_('File changed on disk: %s') % self.path)
 						# T: error message
