@@ -16,10 +16,11 @@ import logging
 
 
 from zim.fs import adapt_from_oldfs
+from zim.parse.encode import url_encode, URL_ENCODE_READABLE
 from zim.newfs import FilePath, LocalFile, LocalFolder
 from zim.notebook import Path, HRef, set_parsetree_attributes_to_resolve_links, \
 	replace_parsetree_links_and_copy_images
-from zim.parsing import is_url_re, url_encode, link_type, URL_ENCODE_READABLE
+from zim.parse.links import is_url_re, link_type
 from zim.formats import get_format, ParseTree, ParseTreeBuilder, \
 	FORMATTEDTEXT, IMAGE, LINK, TAG, OBJECT
 from zim.export.linker import StaticExportLinker
@@ -283,6 +284,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, path=None, text_format
 
 def _link_tree(links, notebook, path):
 	# Convert a list of links (of any type) into a parsetree
+	# To be used on the recieving end of a copy-paste, arguments are the target notebook and path
 	#~ print('LINKS: ', links)
 	#~ print('NOTEBOOK and PATH:', notebook, path)
 	builder = ParseTreeBuilder()
@@ -434,15 +436,27 @@ class InterWikiLinkData(UriData):
 	targets = (PARSETREE_TARGET,) + UriData.targets
 
 	def __init__(self, href, url):
-		UriData.__init__(self, url)
 		self.interwiki_href = href
+		self.interwiki_url = url
+		if self.interwiki_url is not None:
+			UriData.__init__(self, url)
+		else:
+			pass # interwiki may be undefined, resulting in url being None
 
 	def get_data_as(self, targetid):
 		if targetid == PARSETREE_TARGET_ID:
-			parsetree = _link_tree((self.interwiki_href,), None, None)
+			builder = ParseTreeBuilder()
+			builder.start(FORMATTEDTEXT)
+			builder.append(LINK, {'href': self.interwiki_href}, self.interwiki_href)
+			builder.end(FORMATTEDTEXT)
+			parsetree = builder.get_parsetree()
+			parsetree._set_root_attrib('notebook', '-') # force resolve on paste
+			parsetree._set_root_attrib('page', '-')
 			return parsetree.tostring()
-		else:
+		elif self.interwiki_url is not None:
 			return UriData.get_data_as(self, targetid)
+		else:
+			return self.interwiki_href
 
 
 class ParseTreeData(ClipboardData):
@@ -651,10 +665,10 @@ class ClipboardManager(object):
 			if selectiondata:
 				return parsetree_from_selectiondata(selectiondata, notebook, path, text_format)
 			else:
-				logger.warn('Did not get requested data from clipboard')
+				logger.warning('Did not get requested data from clipboard')
 				return None
 		else:
-			logger.warn('Could not paste - no compatible data types on clipboard')
+			logger.warning('Could not paste - no compatible data types on clipboard')
 			return None
 
 	def set_pagelink(self, notebook, path, anchor=None, text=None):
