@@ -10,7 +10,7 @@ import logging
 
 logger = logging.getLogger('zim.notebook.index')
 
-from zim.utils import natural_sort_key
+from zim.base.naturalsort import natural_sort_key
 from zim.notebook.page import Path, HRef, \
 	HREF_REL_ABSOLUTE, HREF_REL_FLOATING, HREF_REL_RELATIVE
 from zim.formats import ParseTreeBuilder
@@ -138,7 +138,7 @@ class PagesIndexer(IndexerBase):
 			file = self.layout.root.file(filerow['path'])
 			format = self.layout.get_format(file)
 			mtime = file.mtime()
-			tree = format.Parser().parse(file.read())
+			tree = format.Parser().parse(file.read(), file_input=True)
 			self.update_page(pagename, mtime, tree)
 		else:
 			pass # some conflict file changed
