@@ -21,14 +21,15 @@ from zim.fs import adapt_from_oldfs
 from zim.newfs import SEP, Folder, LocalFile, LocalFolder
 from zim.config import INIConfigFile, String, ConfigDefinitionByClass, Boolean, Choice
 from zim.errors import Error
-from zim.utils import natural_sort_key
+from zim.base.naturalsort import natural_sort_key
 from zim.newfs.helpers import TrashNotSupportedError
 from zim.config import HierarchicDict
-from zim.parsing import link_type, is_win32_path_re, valid_interwiki_key
+from zim.parse.links import link_type, is_win32_path_re
 from zim.signals import ConnectorMixin, SignalEmitter, SIGNAL_NORMAL
 
+from .info import create_valid_interwiki_key
 from .operations import notebook_state, NOOP, SimpleAsyncOperation, ongoing_operation
-from .page import Path, Page, HRef, HREF_REL_ABSOLUTE, HREF_REL_FLOATING, HREF_REL_RELATIVE
+from .page import Path, Page, PageError, HRef, HREF_REL_ABSOLUTE, HREF_REL_FLOATING, HREF_REL_RELATIVE
 from .index import IndexNotFoundError, LINK_DIR_BACKWARD, ROOT_PATH
 
 DATA_FORMAT_VERSION = (0, 4)
@@ -108,13 +109,6 @@ def _cache_dir_for_dir(dir):
 	return XDG_CACHE_HOME.folder(('zim', path))
 
 
-class PageError(Error):
-
-	def __init__(self, path):
-		self.path = path
-		self.msg = self._msg % path.name
-
-
 class PageNotFoundError(PageError):
 	_msg = _('No such page: %s') # T: message for PageNotFoundError
 
@@ -135,7 +129,7 @@ class PageNotAvailableError(PageNotFoundError):
 		self.file = file
 
 
-class PageExistsError(Error):
+class PageExistsError(PageError):
 	_msg = _('Page already exists: %s') # T: message for PageExistsError
 
 
@@ -366,7 +360,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 			self.icon = None
 		self.document_root = document_root
 
-		self.interwiki = valid_interwiki_key(properties['interwiki'] or self.name)
+		self.interwiki = create_valid_interwiki_key(properties['interwiki'] or self.name)
 
 	def suggest_link(self, source, word):
 		'''Suggest a link Path for 'word' or return None if no suggestion is
@@ -571,7 +565,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 
 			new_n_links = self.links.n_list_links_section(newpath, LINK_DIR_BACKWARD)
 			if new_n_links != n_links:
-				logger.warn('Number of links after move (%i) does not match number before move (%i)', new_n_links, n_links)
+				logger.warning('Number of links after move (%i) does not match number before move (%i)', new_n_links, n_links)
 			else:
 				logger.debug('Number of links after move does match number before move (%i)', new_n_links)
 
@@ -1005,7 +999,12 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		@returns: a L{File} or L{Folder} object.
 		'''
 		assert isinstance(filename, str) and filename
-		file = self._resolve_abs_file(filename)
+		try:
+			file = self._resolve_abs_file(filename)
+		except:
+			logger.exception('Could not resolve filename as absolute path: %s', filename)
+			file = None
+
 		if file is None:
 			if path:
 				folder = self.get_attachments_dir(path)
