@@ -1,23 +1,17 @@
-
 # Copyright 2011 NorfCran <norfcran@gmail.com>
 # License:  same as zim (gpl)
 
-
-
-from gi.repository import Gtk
+from __future__ import annotations
 
 from zim.errors import Error
 from zim.plugins import PluginClass
 from zim.actions import action
-from zim.utils import natural_sort_key
-
+from zim.base.naturalsort import natural_sort_key
 from zim.gui.pageview import PageViewExtension
-from zim.gui.widgets import MessageDialog
 
+#~ import logging
 
-import logging
-
-logger = logging.getLogger('zim.plugins.linesorter')
+#~ logger = logging.getLogger('zim.plugins.linesorter')
 
 
 class LineSorterPlugin(PluginClass):
@@ -131,7 +125,7 @@ class LineSorterPageViewExtension(PageViewExtension):
 			cursor_offset = cursor.get_line_offset()
 
 		# get copy tree
-		tree = buffer.get_parsetree(bounds=(start, end), raw=True)
+		linedata = buffer.get_parsetree(bounds=(start, end))
 
 		with buffer.user_action:
 			# delete lines and insert at target
@@ -143,8 +137,22 @@ class LineSorterPageViewExtension(PageViewExtension):
 			else:
 				iter.backward_lines(abs(offset))
 			buffer.place_cursor(iter)
+
+			# Fixup line end if we move down towards end of buffer without
+			# newline at end of buffer
+			if not iter.starts_line():
+				buffer.insert_at_cursor('\n')
+				iter = buffer.get_insert_iter()
+
+			# actual insert
 			insert_line = iter.get_line()
-			buffer.insert_parsetree_at_cursor(tree)
+			buffer.insert_parsetree_at_cursor(linedata)
+
+			# Fixup line end if selection happened to be end of buffer without newline
+			# and moving up
+			iter = buffer.get_insert_iter()
+			if not iter.starts_line():
+				buffer.insert_at_cursor('\n')
 
 			# redo selection/place cursor at same position
 			if has_selection:
