@@ -1,45 +1,30 @@
 
-# Copyright 2012-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2012-2025 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Pango
+from typing import List, Optional, Tuple
 
-import re
-import datetime
 import logging
 
 logger = logging.getLogger('zim.plugins.tableofcontents')
 
 
 from zim.plugins import PluginClass
-from zim.signals import ConnectorMixin, DelayedCallback
-from zim.notebook import Path
-from zim.tokenparser import collect_untill_end_token, tokens_to_text
+from zim.signals import ConnectorMixin, DelayedCallback, SIGNAL_AFTER
+from zim.parse.tokenlist import tokens_to_text, collect_until_end_token
 from zim.formats import HEADING, LINE
 
+from zim.gui.clipboard import Clipboard, SelectionClipboard
 from zim.gui.pageview import PageViewExtension
 from zim.gui.widgets import LEFT_PANE, PANE_POSITIONS, BrowserTreeView, populate_popup_add_separator, \
 	WindowSidePaneWidget, widget_set_css
-from zim.gui.pageview import FIND_REGEX, SCROLL_TO_MARK_MARGIN, _is_heading_tag, LineSeparatorAnchor
+from zim.gui.pageview import SCROLL_TO_MARK_MARGIN, LineSeparatorAnchor
 
 LINE_LEVEL = 2  # assume level 1 is page heading, level 2 is topic break within page
 
-# FIXME, these methods should be supported by pageview - need anchors - now it is a HACK
-
-def _is_heading_or_line(iter, include_hr):
-	if list(filter(_is_heading_tag, iter.get_tags())):
-		return True
-	elif not include_hr:
-		return False
-	else:
-		anchor = iter.get_child_anchor()
-		if anchor and  isinstance(anchor, LineSeparatorAnchor):
-			return True
-		else:
-			return False
-
 
 def find_heading(buffer, n, include_hr):
 	'''Find the C{n}th heading in the buffer
@@ -47,11 +32,16 @@ def find_heading(buffer, n, include_hr):
 	@param n: an integer
 	@returns: a C{Gtk.TextIter} for the line start of the heading or C{None}
 	'''
+	if include_hr:
+		check = lambda i: buffer.get_line_is_heading(i.get_line()) or buffer.get_anchor_object_at_iter(i, LineSeparatorAnchor)
+	else:
+		check = lambda i: buffer.get_line_is_heading(i.get_line())
+
 	iter = buffer.get_start_iter()
-	i = 1 if _is_heading_or_line(iter, include_hr) else 0
+	i = 1 if check(iter) else 0
 	while i < n:
 		iter.forward_line()
-		while not _is_heading_or_line(iter, include_hr):
+		while not check(iter):
 			if not iter.forward_line():
 				return None
 		i += 1
@@ -69,30 +59,31 @@ def select_heading(buffer, n, include_hr):
 		return False
 
 
-def get_headings(parsetree, include_hr):
-	tokens = parsetree.iter_tokens()
-	stack = [(0, None, [])]
-	for t in tokens:
-		if t[0] == HEADING:
-			level = int(t[1]['level'])
-			text = tokens_to_text(
-						collect_untill_end_token(tokens, HEADING) )
+def get_headings(buffer, include_hr):
+	iter = buffer.get_start_iter()
+	stack = [(0, None, None, [])] # level, text, line, children
+	while True:
+		level, text = buffer.get_heading(iter.get_line())
+		if level is not None: # Is heading
 			assert level > 0 # just to be sure
 			while stack[-1][0] >= level:
 				stack.pop()
-			node = (level, text, [])
-			stack[-1][2].append(node)
+			node = (level, text, iter.get_line(), [])
+			stack[-1][-1].append(node)
 			stack.append(node)
-		elif include_hr and t[0] == LINE:
+		elif include_hr and buffer.get_anchor_object_at_iter(iter, LineSeparatorAnchor):
 			while stack[-1][0] >= LINE_LEVEL:
 				stack.pop()
-			node = (LINE_LEVEL, '\u2500\u2500\u2500\u2500', [])
+			node = (LINE_LEVEL, '\u2500\u2500\u2500\u2500', iter.get_line(), [])
 				# \u2500 == "BOX DRAWINGS LIGHT HORIZONTAL"
-			stack[-1][2].append(node)
+			stack[-1][-1].append(node)
 			stack.append(node)
 		else:
 			pass
 
+		if not iter.forward_line():
+			break
+
 	return stack[0][-1]
 
 
@@ -156,6 +147,7 @@ class ToCPageViewExtension(PageViewExtension):
 
 
 TEXT_COL = 0
+LINE_COL = 1
 
 class ToCTreeView(BrowserTreeView):
 
@@ -180,11 +172,28 @@ class ToCTreeView(BrowserTreeView):
 		if fontsize != 0:
 			self._cell_renderer.set_property('size-points', fontsize)
 
+	def select_heading_for_line(self, line: int):
+		'''Select the heading to which C{line} belongs'''
+		selection = self.get_selection()
+		selection.unselect_all()
+		model = self.get_model()
+
+		previous = None
+		for i in model.walk():
+			if model[i][1] > line: # LINE_COL
+				break
+			else:
+				previous = i
+
+		if previous is not None:
+			treepath = model.get_path(previous)
+			selection.select_path(treepath)
+
 
 class ToCTreeModel(Gtk.TreeStore):
 
 	def __init__(self):
-		Gtk.TreeStore.__init__(self, str) # TEXT_COL
+		Gtk.TreeStore.__init__(self, str, int) # TEXT_COL, LINE_COL
 		self.is_empty = True
 		self.hidden_h1 = False
 
@@ -220,7 +229,7 @@ class ToCTreeModel(Gtk.TreeStore):
 		and len(headings) == 1 \
 		and headings[0][0] == 1:
 			# do not show first heading
-			headings = headings[0][2]
+			headings = headings[0][-1]
 			self.hidden_h1 = True
 		else:
 			self.hidden_h1 = False
@@ -238,10 +247,10 @@ class ToCTreeModel(Gtk.TreeStore):
 
 	def _update_headings(self, headings, parent=None):
 		iter = self.iter_children(parent)
-		for level, text, children in headings:
+		for level, text, line, children in headings:
 			if iter:
 				# Compare to model
-				self[iter] = (text,)
+				self[iter] = (text, line) # TEXT_COL, LINE_COL
 				if children:
 					if self.iter_has_child(iter):
 						self._update_headings(children, iter)
@@ -255,7 +264,7 @@ class ToCTreeModel(Gtk.TreeStore):
 				iter = self.iter_next(iter)
 			else:
 				# Model ran out
-				myiter = self.append(parent, (text,))
+				myiter = self.append(parent, (text, line)) # TEXT_COL, LINE_COL
 				if children:
 					self._insert_headings(children, myiter)
 
@@ -271,8 +280,8 @@ class ToCTreeModel(Gtk.TreeStore):
 				pass
 
 	def _insert_headings(self, headings, parent=None):
-		for level, text, children in headings:
-			iter = self.append(parent, (text,))
+		for level, text, line, children in headings:
+			iter = self.append(parent, (text, line)) # TEXT_COL, LINE_COL
 			if children:
 				self._insert_headings(children, iter)
 
@@ -288,18 +297,20 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 		self.show_h1 = show_h1
 		self.include_hr = include_hr
 		self.fontsize = fontsize
+		self._active_line = None
+		self._connected_to_textbuffer = False # HACK to ensure we initialize correctly
 
 		self.treeview = ToCTreeView(ellipsis, fontsize)
 		self.treeview.connect('row-activated', self.on_heading_activated)
 		self.treeview.connect('populate-popup', self.on_populate_popup)
 		self.add(self.treeview)
 
-		self.connectto(pageview, 'page-changed')
-		self.connectto(pageview.notebook, 'store-page')
-
 		self.pageview = pageview
-		if self.pageview.page:
-			self.load_page(self.pageview.page)
+		self.connectto(pageview, 'page-changed')
+		self.connectto(pageview, 'textbuffer-changed')
+		self.connectto(pageview.notebook, 'store-page')
+		self.on_textbuffer_changed(self.pageview, None, self.pageview.textview.get_buffer())
+		self.on_page_changed(self.pageview, self.pageview.page)
 
 	def set_preferences(self, show_h1, include_hr, fontsize):
 		changed = (show_h1, include_hr, fontsize) != (self.show_h1, self.include_hr, self.fontsize)
@@ -320,13 +331,28 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 
 	def load_page(self, page):
 		model = self.treeview.get_model()
-		tree = page.get_parsetree()
-		if tree is None:
-			model.clear()
+		if model is None:
+			return
+		textview = self.pageview.textview
+		buffer = textview.get_buffer()
+		if buffer.hascontent:
+			model.update(get_headings(buffer, self.include_hr), self.show_h1)
+			self.treeview.select_heading_for_line(buffer.get_insert_iter().get_line())
 		else:
-			model.update(get_headings(tree, self.include_hr), self.show_h1)
+			model.clear()
 		self.emit('changed')
 
+	def on_textbuffer_changed(self, pageview, oldbuffer, newbuffer):
+		if oldbuffer is not None:
+			self.disconnect_from(oldbuffer)
+		self.connectto(newbuffer, 'mark-set', order=SIGNAL_AFTER)
+
+	def on_mark_set(self, textbuffer, iter, mark):
+		# Update selection in widget, but avoid doing it unless line changed
+		if mark.get_name() == 'insert' and iter.get_line() != self._active_line:
+			self._active_line = iter.get_line()
+			self.treeview.select_heading_for_line(self._active_line)
+
 	def on_heading_activated(self, treeview, path, column):
 		self.select_heading(path)
 
@@ -353,14 +379,6 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 		model = self.treeview.get_model()
 		n = model.get_nth_heading(path)
 
-		nextpath = Gtk.TreePath(path[:-1] + [path[-1] + 1])
-		try:
-			aiter = model.get_iter(nextpath)
-		except ValueError:
-			endtext = None
-		else:
-			endtext = model[aiter][TEXT_COL]
-
 		textview = self.pageview.textview
 		buffer = textview.get_buffer()
 		start = find_heading(buffer, n, self.include_hr)
@@ -372,6 +390,29 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 
 		buffer.select_range(start, end)
 
+	def on_copy_link_to_anchor(self, obj, anchor: str, heading_text: str):
+		Clipboard.set_pagelink(self.pageview.notebook, self.pageview.page, anchor, heading_text)
+		SelectionClipboard.set_pagelink(self.pageview.notebook, self.pageview.page, anchor, heading_text)
+		return True
+
+	def can_copy_link_to_anchor(self, paths: List[str]) -> Optional[Tuple[str,str]]:
+		if not paths or len(paths) != 1:
+			return None
+		model = self.treeview.get_model()
+		n = model.get_nth_heading(paths[0])
+		textview = self.pageview.textview
+		buffer = textview.get_buffer()
+		hd_iter = find_heading(buffer, n, self.include_hr)
+		if not hd_iter:
+			return None
+		anchor = buffer.get_anchor_for_location(hd_iter)
+		if not anchor:
+			return None
+		lvl, heading_text = buffer.get_heading(hd_iter.get_line())
+		if not heading_text:
+			return None
+		return anchor, heading_text
+
 	def on_populate_popup(self, treeview, menu):
 		model, paths = treeview.get_selection().get_selected_rows()
 		if not paths:
@@ -381,6 +422,8 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 			can_promote = self.can_promote(paths)
 			can_demote = self.can_demote(paths)
 
+		link_details = self.can_copy_link_to_anchor(paths)
+
 		populate_popup_add_separator(menu, prepend=True)
 		for text, sensitive, handler in (
 			(_('Demote'), can_demote, self.on_demote),
@@ -390,13 +433,33 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 		):
 			item = Gtk.MenuItem.new_with_mnemonic(text)
 			menu.prepend(item)
-			if sensitive:
+			if sensitive and link_details:  # exclude horizontal bars
 				item.connect('activate', handler)
 			else:
 				item.set_sensitive(False)
 
+		menu.append(Gtk.SeparatorMenuItem())
+
+		item = Gtk.MenuItem.new_with_mnemonic(_('Copy _link to this location'))  # T: menu item
+		menu.append(item)
+		if link_details:
+			item.connect('activate', self.on_copy_link_to_anchor, *link_details)
+		else:
+			item.set_sensitive(False)
+
+		# open item in new window
+		item = Gtk.MenuItem.new_with_mnemonic(_('Open in New _Window'))  # T: menu item
+		menu.append(item)
+		if link_details:
+			item.connect('activate', self.on_open_in_new_window, *link_details)
+		else:
+			item.set_sensitive(False)
+
 		menu.show_all()
 
+	def on_open_in_new_window(self, obj, anchor: str, heading_text: str):
+		self.pageview.navigation.open_page(self.pageview.page, anchor, new_window=True)
+
 	def can_promote(self, paths):
 		# All headings have level larger than 1
 		return paths and all(len(p) > 1 for p in paths)
@@ -413,7 +476,7 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 			for i in model.walk(iter):
 				p = model.get_path(i)
 				key = tuple(p)
-				if not key in seen:
+				if key not in seen:
 					if self.show_h1:
 						newlevel = len(p) - 1
 					else:
@@ -474,12 +537,12 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 		if self.select_heading(path):
 			self.pageview.toggle_format('h' + str(level))
 		else:
-			logger.warn('Failed to select heading for path: %', path)
+			logger.warning('Failed to select heading for path: %', path)
 
 
 class SidePaneToC(ToCWidget, WindowSidePaneWidget):
 
-	title = _('ToC') # T: widget label
+	title = _('T_oC') # T: widget label
 
 	def __init__(self, pageview):
 		ToCWidget.__init__(self, pageview, ellipsis=True)
@@ -499,17 +562,17 @@ class MyEventBox(Gtk.EventBox):
 
 class FloatingToC(Gtk.VBox, ConnectorMixin):
 
-	# This class does all the work to keep the floating window in
-	# the right place, and with the right size
+	# This class puts the floating window in the pageview overlay layer
+	# and adjusts it's size on the fly
 
-	X_OFFSET = 10 # offset right side textview
-	Y_OFFSET = 5 # offset top textview
-	S_MARGIN = 5 # margin inside the toc for scrollbars
+	MARGIN_END = 12 # offset right side textview
+	MARGIN_TOP = 12 # offset top textview
+	SCROLL_MARGIN = 10 # margin inside the toc for scrollbars
 
 	def __init__(self, pageview):
 		GObject.GObject.__init__(self)
 
-		self.head = Gtk.Label(label=_('ToC'))
+		self.head = Gtk.Label(label=_('ToC')) # T: small label above Table Of Contents
 		self.head.set_padding(5, 1)
 
 		self.tocwidget = ToCWidget(pageview, ellipsis=False)
@@ -526,23 +589,21 @@ class FloatingToC(Gtk.VBox, ConnectorMixin):
 		widget_set_css(self, 'zim-toc-widget', 'border: 1px solid @fg_color')
 		widget_set_css(self.head, 'zim-toc-head', 'border-bottom: 1px solid @fg_color')
 
-		## Add self to textview
-		# Need to wrap in event box to make widget visible
-		# probably because Containers normally don't have their own
-		# gdk window. So would paint directly on background window.
-		self.textview = pageview.textview
-		self._event_box = MyEventBox()
-		self._event_box.add(self)
+		self.set_halign(Gtk.Align.END)
+		self.set_margin_end(self.MARGIN_END)
+		self.set_valign(Gtk.Align.START)
+		self.set_margin_top(self.MARGIN_TOP)
+		pageview.overlay.add_overlay(self)
 
-		self.textview.add_child_in_window(self._event_box, Gtk.TextWindowType.WIDGET, 0, 0)
-		self.connectto(self.textview,
+		self._textview = pageview.textview
+		self.connectto(self._textview,
 			'size-allocate',
 			handler=DelayedCallback(10, self.update_size_and_position),
 				# Callback wrapper to prevent glitches for fast resizing of the window
 		)
-		self.connectto(self.tocwidget, 'changed', handler=self.update_size_and_position)
+		self.connectto(self.tocwidget, 'changed', handler=self.update_size_and_position_after_change)
 
-		self._event_box.show_all()
+		self.show_all()
 
 	def set_preferences(self, show_h1, include_hr, fontsize):
 		self.tocwidget.set_preferences(show_h1, include_hr, fontsize)
@@ -551,16 +612,16 @@ class FloatingToC(Gtk.VBox, ConnectorMixin):
 		self.tocwidget.disconnect_all()
 		ConnectorMixin.disconnect_all(self)
 
-	def destroy(self):
-		self._event_box.destroy()
-		Gtk.VBox.destroy(self)
-
 	def on_toggle(self, *a):
 		self.tocwidget.set_visible(
 			not self.tocwidget.get_visible()
 		)
 		self.update_size_and_position()
 
+	def update_size_and_position_after_change(self, *a):
+		self.tocwidget.treeview.expand_all()
+		self.update_size_and_position()
+
 	def update_size_and_position(self, *a):
 		model = self.tocwidget.treeview.get_model()
 		if model is None or model.is_empty:
@@ -569,18 +630,18 @@ class FloatingToC(Gtk.VBox, ConnectorMixin):
 		else:
 			self.show()
 
-		text_window = self.textview.get_window(Gtk.TextWindowType.WIDGET)
+		text_window = self._textview.get_window(Gtk.TextWindowType.WIDGET)
 		if text_window is None:
 			return
 
 		text_x, text_y, text_w, text_h = text_window.get_geometry()
-		max_w = 0.5 * text_w - self.X_OFFSET
-		max_h = 0.7 * text_h - self.Y_OFFSET
+		max_w = 0.5 * text_w - self.MARGIN_END
+		max_h = 0.7 * text_h - self.MARGIN_TOP
 
 		head_minimum, head_natural = self.head.get_preferred_width()
 		view_minimum, view_natural = self.tocwidget.treeview.get_preferred_width()
 		if self.tocwidget.get_visible():
-			my_width = max(head_natural, view_natural + self.S_MARGIN)
+			my_width = max(head_natural, view_natural + self.SCROLL_MARGIN)
 			width = min(my_width, max_w)
 		else:
 			width = head_natural
@@ -588,13 +649,9 @@ class FloatingToC(Gtk.VBox, ConnectorMixin):
 		head_minimum, head_natural = self.head.get_preferred_height()
 		view_minimum, view_natural = self.tocwidget.treeview.get_preferred_height()
 		if self.tocwidget.get_visible():
-			my_height = head_natural + view_natural + self.S_MARGIN
+			my_height = head_natural + view_natural + self.SCROLL_MARGIN
 			height = min(my_height, max_h)
 		else:
 			height = head_natural
 
 		self.set_size_request(width, height)
-
-		x = text_w - width - self.X_OFFSET
-		y = self.Y_OFFSET
-		self.textview.move_child(self._event_box, x, y)
