@@ -12,11 +12,11 @@ import zim.datetimetz as datetime
 from zim.notebook import Path
 from zim.notebook.index.base import IndexerBase, IndexView
 from zim.notebook.index.pages import PagesViewInternal
-from zim.formats import get_format, \
-	UNCHECKED_BOX, CHECKED_BOX, XCHECKED_BOX, MIGRATED_BOX, TRANSMIGRATED_BOX, BULLET, TAG, ANCHOR, \
-	HEADING, PARAGRAPH, BLOCK, NUMBEREDLIST, BULLETLIST, LISTITEM, STRIKE
-from zim.tokenparser import TEXT, END, \
-	skip_to_end_token, tokens_to_text, tokens_by_line, collect_until_end_token
+from zim.parse.tokenlist import TEXT, END, collect_until_end_token, skip_to_end_token, tokens_by_line
+from zim.formats import \
+	UNCHECKED_BOX, CHECKED_BOX, XCHECKED_BOX, MIGRATED_BOX, TRANSMIGRATED_BOX, TAG, ANCHOR, \
+	HEADING, PARAGRAPH, NUMBEREDLIST, BULLETLIST, LISTITEM, STRIKE
+
 
 from zim.plugins.journal import daterange_from_path
 	# TODO instead of just importing this function we should define
@@ -25,14 +25,11 @@ from zim.plugins.journal import daterange_from_path
 logger = logging.getLogger('zim.plugins.tasklist')
 
 
-from zim.parsing import parse_date as old_parse_date
-from .dates import date_re as _raw_parse_date_re
-from .dates import parse_date
-
+from zim.parse.dates import date_re as _raw_parse_date_re
+from zim.parse.dates import parse_date, old_parse_date
 
 _tag_re = re.compile(r'(?<!\S)@(\w+)\b', re.U)
-_day_re = re.compile('(\d{1,2})')
-_date_re = re.compile('[<>] ?' + _raw_parse_date_re.pattern + '|\[d:.+\]')
+_date_re = re.compile('[<>] ?' + _raw_parse_date_re.pattern + r'|\[d:.+\]')
 	# "<" and ">" prefixes for dates, "[d: ...]" for backward compatibility
 
 _MIN_START_DATE = '0'
@@ -88,7 +85,7 @@ def _parse_page_list(input):
 		try:
 			paths.append(Path(Path.makeValidPageName(name.strip())))
 		except ValueError:
-			logger.warn('Could not parse page name: "%s"', name)
+			logger.warning('Could not parse page name: "%s"', name)
 	return paths
 
 
@@ -722,12 +719,12 @@ class TaskParser(object):
 
 					next_token = next(token_iter)
 					if not next_token == (END, LISTITEM):
-						logger.warn('Unexpected token in list: %r', next_token)
+						logger.warning('Unexpected token in list: %r', next_token)
 
 			elif t[0] == END and t[1] in (BULLETLIST, NUMBEREDLIST):
 				break
 			else:
-				logger.warn('Unexpected token in list: %r', t)
+				logger.warning('Unexpected token in list: %r', t)
 
 		return tasks
 
@@ -776,9 +773,9 @@ class TaskParser(object):
 				elif string.startswith('<'):
 					due = parse_date(string[1:]).last_day.isoformat()
 				else:
-					logger.warn('False positive matching date: %s', string)
+					logger.warning('False positive matching date: %s', string)
 			except ValueError:
-				logger.warn('Invalid date format in task: %s', string)
+				logger.warning('Invalid date format in task: %s', string)
 
 		return [status, prio, waiting, start, due, tags, str(text.strip())]
 			# 0:status, 1:prio, 2:waiting, 3:start, 4:due, 5:tags, 6:desc
