@@ -11,7 +11,7 @@ from typing import Generator, Generic, List, Optional, Union
 logger = logging.getLogger('zim.notebook')
 
 
-from zim.parsing import link_type
+from zim.parse.links import link_type
 from zim.errors import Error
 
 import zim.formats
@@ -25,7 +25,7 @@ import zim.datetimetz as datetime
 _pagename_reduce_colon_re = re.compile('::+')
 _pagename_invalid_char_re = re.compile(
 	'(' +
-		'^[_\W]+|(?<=:)[_\W]+' +
+		r'^[_\W]+|(?<=:)[_\W]+' +
 	'|' +
 		'[' + re.escape(''.join(
 			("?", "#", "/", "\\", "*", '"', "<", ">", "|", "%", "\t", "\n", "\r")
@@ -350,7 +350,7 @@ class HRef():
 		return klass.new_from_wiki_link(href).to_wiki_link()
 
 	@classmethod
-	def new_from_wiki_link(klass, href: str) -> 'Href':
+	def new_from_wiki_link(klass, href: str) -> 'HRef':
 		'''Constructor that constructs a L{HRef} object for a link as
 		written in zim's wiki syntax.
 		@param href: a string for the link
@@ -360,6 +360,8 @@ class HRef():
 		@note: This method HRef class assumes the logic of our wiki links.
 		For other formats, a separate constructor may be needed.
 		'''
+		href = href.strip()
+
 		if href.startswith(':'):
 			rel = HREF_REL_ABSOLUTE
 		elif href.startswith('+'):
@@ -411,7 +413,14 @@ class HRef():
 		return link
 
 
-class PageReadOnlyError(Error):
+class PageError(Error):
+
+	def __init__(self, path):
+		self.path = path
+		self.msg = self._msg % path.name
+
+
+class PageReadOnlyError(PageError):
 	_msg = _('Can not modify page: %s') # T: error message for read-only pages
 
 
@@ -517,7 +526,7 @@ class Page(Path, SignalEmitter):
 		# for saving and is reset after save was done
 		if buffer.get_modified():
 			if self.readonly:
-				logger.warn('Buffer edited while page read-only - potential bug')
+				logger.warning('Buffer edited while page read-only - potential bug')
 			self.set_modified(True)
 
 	def _store(self):
