@@ -31,19 +31,17 @@ to let plugins extend specific application objects.
 
 
 from gi.repository import GObject
-import types
-import os
-import sys
 import logging
-import inspect
 import weakref
 
 import collections.abc as abc
 
+from zim.base import classproperty
+from zim.base.klasslookup import get_module, lookup_subclass, lookup_subclasses
+
 from zim.newfs import LocalFolder, LocalFile
 
-from zim.signals import SignalEmitter, ConnectorMixin, SIGNAL_AFTER, SIGNAL_RUN_LAST, SignalHandler
-from zim.utils import classproperty, get_module, lookup_subclass, lookup_subclasses
+from zim.signals import SignalEmitter, ConnectorMixin, SIGNAL_RUN_LAST
 from zim.actions import hasaction, get_actions
 
 from zim.config import data_dirs, XDG_DATA_HOME, ConfigManager
@@ -178,6 +176,19 @@ def list_actions(obj):
 	return actions
 
 
+def get_plugin_key(obj):
+	'''Tries to determine the plugin key for C{obj}
+	Derives the name from C{__module__}
+	Raises C{AssertionError} on failure
+	'''
+	parts = obj.__module__.split('.')
+	if 'plugins' in parts:
+		i = parts.index('plugins')
+		return parts[i+1]
+	else:
+		raise AssertionError('Cannot determine plugin for %s, (%s)' % (obj, obj.__module__))
+
+
 class ExtensionBase(SignalEmitter, ConnectorMixin):
 	'''Base class for all extensions classes
 	@ivar plugin: the plugin object to which this extension belongs
@@ -321,7 +332,7 @@ class InsertedObjectTypeMap(SignalEmitter):
 		return [k for k in self]
 
 	def items(self):
-		return [(k, self[v]) for k in self]
+		return [(k, self[k]) for k in self]
 
 	def values(self):
 		return [self[k] for k in self]
@@ -442,7 +453,7 @@ class PluginManagerClass(ConnectorMixin, SignalEmitter, abc.Mapping, metaclass=_
 		# List "zim.plugins" sub modules based on __path__ because this
 		# parameter determines what folders will considered when importing
 		# sub-modules of the this package once this module is loaded.
-		plugins = set(['arithmetic', 'attachmentbrowser', 'backlinkpane', 'bookmarksbar', 'commandpalette', 'diagrameditor', 'distractionfree', 'ditaaeditor', 'equationeditor', 'gnu_r_ploteditor', 'gnuplot_ploteditor', 'inlinecalculator', 'insertsymbol', 'journal', 'linesorter', 'linkmap', 'osx_menubar', 'pageindex', 'pathbar', 'printtobrowser', 'quicknote', 'scoreeditor', 'screenshot', 'sequencediagrameditor', 'sourceview', 'spell', 'tableeditor', 'tableofcontents', 'tags', 'tasklist', 'toolbar', 'trayicon', 'versioncontrol', 'zeitgeist-logger']) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY
+		plugins = set(['arithmetic', 'attachmentbrowser', 'backlinkpane', 'bookmarksbar', 'commandpalette', 'diagrameditor', 'distractionfree', 'ditaaeditor', 'equationeditor', 'gnu_r_ploteditor', 'gnuplot_ploteditor', 'indexed_fts', 'inlinecalculator', 'insertsymbol', 'journal', 'linesorter', 'linkmap', 'mermaiddiagrameditor', 'osx_menubar', 'pageindex', 'pathbar', 'printtobrowser', 'quicknote', 'scoreeditor', 'screenshot', 'sequencediagrameditor', 'sourceview', 'spell', 'tableeditor', 'tableofcontents', 'tags', 'tasklist', 'toolbar', 'trayicon', 'versioncontrol', 'windowtitleeditor', 'zeitgeist-logger']) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY
 		for folder in [f for f in map(LocalFolder, __path__) if f.exists()]:
 			for child in folder:
 				name = child.basename
@@ -778,3 +789,14 @@ class PluginClass(ConnectorMixin):
 		Can be implemented by sub-classes.
 		'''
 		pass
+
+	def show_preferences(self, widget):
+		'''Convenience method to show the preferences dialog for this plugin
+		@param widget: C{Gtk.Widget} used to find toplevel window
+		'''
+		mykey = get_plugin_key(self)
+		window = widget.get_toplevel()
+		if not window:
+			raise AssertionError('Need toplevel which supports "show_preferences" action - no window')
+		action = find_action(window, 'show_preferences')
+		action(select_plugin=mykey)
